#include "header.h"
#include "html.h"

// comparing function for 2 BS objects, decreasing
bool BS::BS_compare(BS *bs1, BS *bs2) {
  return (bs1->score > bs2->score);
}

void to_color_string(const char * motif_str, const int motif_len, string &str){
  str = "";
  for (int i=0; i<motif_len; i++) {
    switch( motif_str[i] ) {
    case 'A':
      str += "<font class='style_A'>A </font>";
      //str += color_A;
      break;
    case 'C':
      str += "<font class='style_C'>C </font>";
      //str += color_C;
      break;
    case 'G':
      str += "<font class='style_G'>G </font>";
      //str += color_G;
      break;
    case 'T':
      str += "<font class='style_T'>T </font>";
      //str += color_T;
      break;
    default:
      str += "<font class='style__'><b>?</b></font>";
    }
  }
}

HTML_output::HTML_output (FindMotif &_fm,  const int out_size) :
  fm(_fm),
  motif_len(fm.get_motif_len()),
  num_true_seq(fm.get_num_true_seq()),
  num_false_seq(fm.get_num_false_seq()),
  output_size (out_size),
  true_dna_vec(fm.get_true_dna_vec()),
  false_dna_vec(fm.get_false_dna_vec())
{ };

void HTML_output::output(vector<double*> &seedVec, ostream &os) {
  //cerr << " FindMotif::output_HTML( vector<double*> &seedVec, ostream &os)" << endl;
  
  os.width(4);
  os.precision(3);

  int outputSize = seedVec.size();
  if (outputSize > FindMotif::Num_Motif)
    outputSize = FindMotif::Num_Motif;
  if (outputSize > output_size)
    outputSize = output_size;
  char motif_str[motif_len+1];
  
  os << "<h2>Found Motifs:</h2><a name='all_motifs'>" << endl;
  os << "<table>" << endl
     << "  <tr><td width='200' class='headerrow'>&nbsp;</td>" << endl
     << "    <td width='200' class='headerrow'>Pattern</td>" <<endl
     << "    <td width='150' class='headerrow'><i>p</i>-value</td>" << endl
     << "    <td width='150' class='headerrow'>Alpha (&#945;)</td>" << endl
     << "  </tr>" << endl;

  for (int i=0; i<outputSize; i++) {
    Motif *motif = fm.get_motif(i);
    os << "  <tr class="<< (i%2 ==0 ? "'evenrow'" : "'oddrow'" ) << "><td align='center'><b>"
       << "<a href='#motif" << i 
       << "'>Motif #" <<  (i+1) << "</b>:</b></td><td align='center'>";
    
    Tools::matToStr(fm.get_motif(i)->getProbMat(), motif_str, motif_len);
    fm.get_motif(i)->set_pattern(motif_str);
    string color_str;
    to_color_string(motif_str, motif_len, color_str);
    os << color_str << "</td>" << endl;
    os << "<td align='center'>" << motif->getScore()
       << "</td><td align='center'>" << exp(motif->getAlpha()) << endl 
       << "</td></tr>" << endl;
  }
  os << "</table>" << endl;
  

  os << "<hr/>" << endl;


  // output result for each motif
  for(int i=0; i<outputSize; i++) {   
    Motif *motif = fm.get_motif(i);
    const double * prob_mat = motif->getProbMat();

    os << "<a name=\"motif"<< i << "\"></a><h3>Motif #"
       << (i+1) << "</h3>" << endl;

    string color_str;
    to_color_string( motif->get_pattern().c_str(), motif_len, color_str);

    os << "<table width='500'>" << endl;
    os << "  <tr class='oddrow'><td width='150' class='headerrow'>Pattern</td><td><nobr><font size='+2'>&nbsp;&nbsp;" << color_str << "</font></nobr></td></tr>" << endl;
    os << "  <tr class='evenrow'><td width='150' class='headerrow'><i>p</i>-value</td><td><font color='darkblue' size='+2'><b>&nbsp;&nbsp;" << motif->getScore() << "</b></font></td></tr>" << endl;
    os << "  <tr class='oddrow'><td width='150' class='headerrow'>Alpha (&#945;)</td><td><font color='darkblue' size='+2'><b>&nbsp;&nbsp;" << exp(motif->getAlpha()) << "</b></font></td></tr>" << endl;
    os << "</table><p>" << endl;

    output_Prob(prob_mat, os);
    output_BS(prob_mat, motif->getAlpha(), os);
  }
}

void HTML_output::output_Prob(const double *prob_mat, ostream &os) {
  os << "<h4>Probility Matrix:</h4>" << endl
     << "<table>" << endl
     << "  <tr><td width='70' align='center' class='headerrow'>&nbsp;</td>" << endl;
  for(int k=0; k<motif_len; k++)   {
    os << "<td align='center' class='headerrow' width='45'><i>#"<< (k+1) <<"</i></td>" << endl;
  }
  // row A
  os << "  </tr>\n  <tr class='oddrow'><td width='50' align='center'><font class='style_A'>A</font></td>" << endl;
  for(int k=0; k<motif_len; k++) {
    double prob = prob_mat[k];
    os << "<td align='center'>";
    if (prob >= prob_mat[k+motif_len] &&
	prob >= prob_mat[k+motif_len*2] &&
	prob >= prob_mat[k+motif_len*3] ) {
      os << "<font color='darkred'><b>" << prob << "</b></font>";
    } else {
      os << "<font color='grey'><i>" << prob << "</i></font>";
    }
    os << "</td>" << endl;
  }
  // row C
  os << "  </tr>\n  <tr class='evenrow'><td width='50' align='center'><font class='style_C'>C</font></td>" << endl;
  for(int k=0; k<motif_len; k++) {
    double prob = prob_mat[k+motif_len];
    os << "<td align='center'>";
    if (prob>=prob_mat[k] &&
	prob>=prob_mat[k+motif_len*2] &&
	prob>=prob_mat[k+motif_len*3] ) {
      os << "<font color='darkred'><b>" << prob << "</b></font>";
    } else {
      os << "<font color='grey'><i>" << prob << "</i></font>";
    }
    os << "</td>" << endl;
  }
  // row G
  os << "  </tr>\n  <tr class='oddrow'><td width='50' align='center'><font class='style_G'>G</font></td>" << endl;
  for(int k=0; k<motif_len; k++) {
    double prob = prob_mat[k+2*motif_len];
    os << "<td align='center'>";
    if (prob>=prob_mat[k] &&
	prob>=prob_mat[k+motif_len] &&
	prob>=prob_mat[k+motif_len*3] ) {
      os << "<font color='darkred'><b>" << prob << "</b></font>";
    } else {
      os << "<font color='grey'><i>" << prob << "</i></font>";
    }
    os << "</td>" << endl;
  }
  // row T
  os << "  </tr>\n  <tr class='evenrow'><td width='50' align='center'><font class='style_T'>T</font></td>" << endl;
  for(int k=0; k<motif_len; k++) {
    double prob = prob_mat[k+3*motif_len];
    os << "<td align='center'>";
    if (prob>=prob_mat[k] &&
	prob>=prob_mat[k+motif_len] &&
	prob>=prob_mat[k+motif_len*2] ) {
      os << "<font color='darkred'><b>" << prob << "</b></font>";
    } else {
      os << "<font color='grey'><i>" << prob << "</i></font>";
    }
    os << "</td>" << endl;
  }
  os << "  </tr>" << endl;
  os << "</table>" << endl;
}

double calc_score (const char * seq,
	      const double *prob_mat,
	      const int motif_len) {
  double score = 0.0;
  for (int k=0; k<motif_len; k++) {
    switch(seq[k]) {
    case 'A':
      score += log (prob_mat[k]);
      break;
    case 'C':
      score += log (prob_mat[k+motif_len]);
      break;
    case 'G':
      score += log (prob_mat[k+2*motif_len]);
      break;
    case 'T':
      score += log (prob_mat[k+3*motif_len]);
      break;
    default:
      exit(1);
    }
  }
  return score;
}

void HTML_output::output_BS(const double *prob_mat, const double &alpha,
			    ostream &os) {
  os << "<h4>Pattern appearing in the sequences:</h4><table>" << endl;
  os << "  <tr><td width='120' class='headerrow'><nobr>Sequence Name&nbsp;&nbsp;</nobr></td><td width='80' class='headerrow'><nobr>Score</nobr>&nbsp;&nbsp;</td><td sidth='80' class='headerrow' align='center'><nobr>Pattern range</nobr></td><td align='center' colspan='3' class='headerrow'>Pattern</td></tr>";

  // iterate all sequences
  bool is_odd_row = true;
  for ( vector<DNA*>::iterator i=true_dna_vec.begin();
        i < true_dna_vec.end();
        i++) {
    DNA * dna = (*i);
    const char * seq = dna->getSeq();
    const char * seqR = dna->getSeqR();
    const int seqlen = strlen(seq);
    int right_most = seqlen-motif_len;
    
    vector<BS *> all_substr;

    // iterate all substr sequences, string at j-th positions
    for (int j=0; j<=right_most; j++) {
      // calculate score of seq[j]...seq[j+motif_len-1]
      //int end = j+motif_len;
      double score = calc_score(seq+j, prob_mat, motif_len);
      if (score >= alpha) {
	all_substr.push_back(new BS( 0 , seq, j, true, score) );
      }
    }

    // reversed string
    // iterate substr sequences
    if (FindMotif::Get_Test_Rev() == true) {
      for (int j=0; j<=right_most; j++) {
	// calculate score of seqR[j]...seqR[j+motif_len-1]
	//int end = j+motif_len;
	double score =  calc_score(seqR+j, prob_mat, motif_len);

	if (score >= alpha) {
	  all_substr.push_back(new BS( 0 , seqR, j, false, score) );
	}
      }

    }

    // for EACH true sequence, sort all binding site, select the highest scores
    // sort(...)
    output_BS2( dna, seq, all_substr, os, is_odd_row);

    //delete
    for ( vector<BS *>::iterator i=all_substr.begin();
	  i < all_substr.end();
	  i++) {
      delete (*i);
    }
    all_substr.clear();
  }
  os << "</table><p/><div align='right'><font size='-1'>[<a href=\"#all_motifs\">back</a>]</font></div><hr/>";
}

void HTML_output::output_BS2( DNA *dna, const char *seq,
			      vector<BS *> &all_substr,
			      ostream &os, bool &is_odd) {
  int seqlen = strlen(seq);
  sort(all_substr.begin(), all_substr.end(), BS::BS_compare);

  int num=0;
  for ( vector<BS *>::iterator i=all_substr.begin();
        i < all_substr.end();
        i++) {
    if ( ++num >  fm.get_max_binding_ratio() ) {
      return;
      // stop outputting
    }

    BS *bs = (*i);
    const char* tempseq = bs->seq;

    //fprintf(stderr, "%s --> seq[%d..%d], score=%f\n",dna->getName(), j, j+motif_len-1, bs.scors);
    const char * style_str = is_odd ? "oddrow" : "evenrow";
    is_odd = !is_odd;
    os << "<tr class='"<< style_str << "'><td>" << dna->getName();
    // forward??? or reversed??
    if (!bs->forward) os << " <font colo='darkred' size='-1'><b><i>(R)</i></b></font>";

    os << "</td><td align='center'>" << exp(bs->score) << "</td>";
    
    os << "<td align='center'><font size='-1'>[<i>";
    if ( bs->forward ) {
      os<< 1+bs->pos << ".." << bs->pos+motif_len;
    } else {
      os<< seqlen - bs->pos << ".." << seqlen - (bs->pos+motif_len-1);
      //os<< bs->pos+motif_len-1 << ".." << bs->pos;
    }
    os << "</i>]</font></td><td align='right' class='style__'><b>&nbsp;&nbsp;";
    //output substring before the binding sites
    for (int i=bs->pos-8; i<bs->pos; i++) {
      if (i>=0) os << tempseq[i];
    }
    os << "&nbsp;</b></td><td align='center'><nobr>&nbsp;" << endl;
    
    for (int i=bs->pos; i<bs->pos+motif_len; i++) {
      switch(tempseq[i]) {
      case 'A':
	os << "<font class='style_A'>A</font>";
	//os << color_A;
	break;
      case 'C':
	os << "<font class='style_C'>C</font>";
	//os << color_C;
	break;
      case 'G':
	os << "<font class='style_G'>G</font>";
	//os << color_G;
	break;
      case 'T':
	os << "<font class='style_T'>T</font>";
	//os << color_T;
	break;
      default:
	exit(1);
      }
      os << " ";
    }
    os << "&nbsp;</nobr></td><td align='left' class='style__'><b>&nbsp;";
    //output substring after the binding sites
    for (int i=bs->pos+motif_len; i<bs->pos+motif_len+8; i++) {
      if (i<seqlen) os << seq[i];
    }
    os << "</b>&nbsp;</td></tr>";
  }
}
