#ifndef MOTIF_H
#define MOTIF_H
#include "header.h"

class Motif {
public:
  Motif(double* probMat, const double score, const int* bsDistributeT
        , const int* bsDistributeF, const double alpha);	//constructor
  ~Motif();						//destructor

  Motif(const char* pat, const short scr) :
    pattern(pat),
    score(scr) {};

  
/*****************************************************************************/
/*                                                                           */
/* Pub 1 Access Attributes                                                   */
/*                                                                           */
/*****************************************************************************/

  //get the probability matrix of the motif
  const double* getProbMat();
  
  //get the score of the motif
  double getScore();
  
  //get the distribution of the binding sites in true set of the motif
  const int* getBSDistributeT();
  
  //get the distribution of the binding sites in false set of the motif
  const int* getBSDistributeF();
  
  //get the threshold alpha of the motif
  double getAlpha();
  
  string & get_pattern () {
    return pattern;
  }

  void set_pattern ( char * pat ) {
    pattern = pat;
  }

  void set_pattern ( string &pat ) {
    pattern = pat;
  }

  void set_score ( const double &scr ) {
    score = scr;
  }

  static int sortFunc( const void *a, const void *b);

  private:
/*****************************************************************************/
/*                                                                           */
/* Pri 1 parameters                                                          */
/*                                                                           */
/*****************************************************************************/
  double* probMat;
  const int* bsDistributeT;
  const int* bsDistributeF;
  double alpha;
  
  string pattern;
  double score;
};

#endif

