#include "header.h"

/*****************************************************************************/
/*                                                                           */
/* Constructor and Destructor                                                */
/*                                                                           */
/*****************************************************************************/
Tools::Tools(const int _T, const int _F, const double _m,
	     const int _w, const int _n) :
  T (_T),
  F (_F),
  maxBindRatio (_m),
  motifLength (_w),
  seqLength (_n),
  noOfRowT (_T*(int)maxBindRatio+1),
  noOfColF (_F*(int)maxBindRatio+1),
  expectArr (new long double[noOfRowT * noOfColF])
{
/////////////////////////////////////////////////////////////////////
  cout<<"Construct Tools"<<endl;
  cout<<"noOfRowT = "<<noOfRowT<<", noOfColF = "<< noOfColF<<endl;
  cout<<endl;
  
  
  for(int i=0;i<noOfRowT * noOfColF;i++)
    this->expectArr[i]=0.0;

  char progress[4] = {'-','\\', '|', '/'};
  int status=0;  //deallocating memory
    
  //P_s
  long double* bsInSeqArr=new long double[noOfRowT]; 
  //p_s2
  long double* bsInSeqArr2=new long double[noOfRowT];
  
  //P_t
  long double* bsInT=new long double[noOfRowT]; 
  //P_f
  long double* bsInF=new long double[noOfColF];
  
#define MAX_SEED_LENGTH 6

  int step=1;

  if ( motifLength > MAX_SEED_LENGTH) {
    step = (int) pow(4.0, motifLength-MAX_SEED_LENGTH);
    //fprintf(stderr, "\nSetting step to %d (approximate)\n\n", step);
  }

  fprintf(stderr, "\t");

  int pow_4 = (int)pow(4.0,motifLength);

  for(int alpha=step; alpha<=pow_4; alpha+=step) {
    //for(int alpha=(int)pow(4.0,motifLength)/2;alpha<=(int)pow(4.0,motifLength)/2;alpha+=step) {
    //for(int alpha=4;alpha<=4;alpha+=step) {    
    //true seq
    for(int B=0;B<=(int)maxBindRatio*T;B++) {    
      bsInSeqArr[B]=Tools::pS(alpha, motifLength, B, seqLength*T);
      bsInSeqArr2[B]=Tools::pS2(alpha, motifLength, B, seqLength*T);
      bsInT[B]=bsInSeqArr[B]-bsInSeqArr2[B];

    }                                          
    
    //false seq                                      
    for(int B=0;B<=(int)maxBindRatio*F;B++) {    
      bsInF[B]=Tools::pS(alpha, motifLength, B, seqLength*F);
    } 
    for(int t=0;t<=T*(int)maxBindRatio;t++) {
      for(int f=0;f<=F*(int)maxBindRatio;f++) {  
        expectArr[t*noOfColF+f]+=bsInT[t]*bsInF[f]*step;
      }
    }
    fprintf(stderr, "%c\b", progress[status++]);
    status = status % 4;
  }//end of for all alpha
  
  //calculate expectArr
  for(int i=T*(int)maxBindRatio;i>=0; i--) {
    for(int j=0;j<=F*(int)maxBindRatio;j++) {
      double temp=expectArr[i*noOfColF+j];
      
      if(i<T*(int)maxBindRatio)
        temp+=expectArr[(i+1)*noOfColF+j];
      if(j>0)
        temp+=expectArr[i*noOfColF+j-1];
      if(i<T*(int)maxBindRatio && j>0)
        temp-=expectArr[(i+1)*noOfColF+j-1];
      expectArr[i*noOfColF+j]=temp;
    }
    fprintf(stderr, "%c\b", progress[status++]);
    status = status % 4;
  }
  delete [] bsInSeqArr; 
  delete [] bsInSeqArr2;
  delete [] bsInT;
  delete [] bsInF;
}

Tools::~Tools() {
  if(expectArr!=NULL)
    delete [] expectArr;
}

/*****************************************************************************/
/*                                                                           */
/* Pub 1 Sorting                                                             */
/*                                                                           */
/*****************************************************************************/

//sorting the double array inArr in decreasing order
void Tools::sortDouble_D(double* inArr, const int arraySize) {
  double* tempArr=new double[arraySize];
  if(tempArr==NULL){
    cout<<"In Tools::sortDouble_D"<<endl;
    cout<<"No enough memory for tempArr"<<endl;
    exit(1);
  }
  
  double* sortedArr=Tools::sortDouble_D_M(inArr,tempArr,0,arraySize);
  
  if(sortedArr != inArr) {
    for(int i=0;i<arraySize;i++) {
      inArr[i]=sortedArr[i];
    }
  }
  delete [] tempArr;
}

//sorting the BindingSitePosition array inArr in decreasing order
void Tools::sortBSP_D(BindingSitePosition** inArr, const int arraySize) {
  BindingSitePosition** tempArr=new BindingSitePosition*[arraySize];
  if(tempArr==NULL){
    cout<<"In Tools::sortBSP_D"<<endl;
    cout<<"No enough memory for tempArr"<<endl;
    exit(1);
  }
  
  BindingSitePosition** sortedArr=Tools::sortBSP_D_M(inArr,tempArr,0,arraySize);
  
  if(sortedArr != inArr) {
    for(int i=0;i<arraySize;i++) {
      inArr[i]=sortedArr[i];
    }
  }
  delete [] tempArr;
}
/*****************************************************************************/
/*                                                                           */
/* Pub 2 String Operation Methods                                            */
/*                                                                           */
/*****************************************************************************/

//convert an integer to a string
string Tools::intToStr(int i) {
  ostringstream oss;
  oss << i;
  return oss.str();
}

//convert a double to a string  
string Tools::dblToStr(double d) {
  ostringstream oss;
  oss << d;
  return oss.str();
}
  
//split a string str into pieces
void Tools::split(string str, char delimiter, vector<string> &tokenVec) {
  
  string token="";
  
  for(int i=0;i < (int)str.length();i++) {
    if(str[i]==delimiter) {
      tokenVec.push_back(token);
      token="";
    }
    else {
      token+=str[i];
    }
  }
  tokenVec.push_back(token);
}

//convert a probability matrix to string
void Tools::matToStr(const double *probMat, char* motif, const int motifLength) {
  char c;
  double maxProb;
  for(int i=0;i<motifLength;i++) {
    c='A';
    maxProb=probMat[i];
    if(probMat[i+motifLength]>maxProb) {
      c='C';
      maxProb=probMat[i+motifLength];
    }
    if(probMat[i+2*motifLength]>maxProb) {
      c='G';
      maxProb=probMat[i+2*motifLength];
    }
    if(probMat[i+3*motifLength]>maxProb) {
      c='T';
      maxProb=probMat[i+3*motifLength];
    }
    motif[i]=c;
    if(maxProb==0.25)
      motif[i]='N';
  }
  motif[motifLength]=0;
}

//convert a number to probability matrix
void Tools::numToMat(const int num, double* probMat, const int motifLength, const int seedLength) {
  int preSpace=(motifLength-seedLength)/2;
    
  for(int i=0;i<4*motifLength;i++)
    probMat[i]=0.25;
  
  for(int i=0;i<4;i++) {
    for(int j=0;j<seedLength;j++) {
      probMat[i*motifLength+preSpace+j]=0.5/3;
    }
  }

  int value=num;	//get the pattern number;
  for(int i=0;i<seedLength;i++) {
    switch(value%4) {
      case 0:
        probMat[preSpace+i]=0.5;
        break;
      case 1:
        probMat[motifLength+preSpace+i]=0.5;
        break;
      case 2:
        probMat[2*motifLength+preSpace+i]=0.5;
        break;
      case 3:
        probMat[3*motifLength+preSpace+i]=0.5;
        break;
      default:
        cout<<"In Tools::numToMat"<<endl;
        cout<<"Error in switch, value = "<<value<<", value%4 = "<<value%4<<endl;
        exit(1);
    }
    value/=4;
  }
}
//construct an array of all string of length strinfLength
void Tools::allString(char** stringArr, const int stringLength) {
  int arraySize=(int)pow(4.0,stringLength);
  
  for(int i=0;i<arraySize;i++) {
    int value=i;
    stringArr[i]=new char[stringLength+1];
    for(int j=0;j<stringLength;j++) {
      switch(value%4) {
        case 0:
          stringArr[i][j]='A';
          break;
        case 1:
          stringArr[i][j]='C';
          break;
        case 2:
          stringArr[i][j]='G';
          break;
        case 3:
          stringArr[i][j]='T';
          break;
        default:
          cout<<"In Tools::allString"<<endl;
          cout<<"Error in switch, value = "<<value<<", value%4 = "<<value%4<<endl;
          exit(1);
      }
      value=value/4;
    }
    stringArr[i][stringLength]=0;
  }
}

/*****************************************************************************/
/*                                                                           */
/* Pub 3 probability of a random matrix with higer t and lower f             */
/*                                                                           */
/*****************************************************************************/
//calculate the score
double Tools::score(const int t, const int f) {
  if(t<noOfRowT && f<noOfColF)
    return (double)expectArr[t*noOfColF+f];
  else {
    cout<<"t = "<<t<<" >= noOfRowT = "<<noOfRowT<<" or f = "<<f<<" >= noOfColF = "<<noOfColF<<endl;
    exit(1);
  }
}
  
//calculate the p_d
long double Tools::pAlpha(const int alpha, const int w) {
  return ((long double)alpha)/pow(4.0,w);
}

//calculate P_s
long double Tools::pS(const int alpha, const int w, const int B, const int n) {    
  long double p_alpha=Tools::pAlpha(alpha,w);
  long double OneMinusPa=1.0-p_alpha;
  
  if(B==0)
    return pow(OneMinusPa,n-w+1);
  if(B>n/w)
    return 0.0;
/*  
    C(b-1, b-1) + C(b,b-1) + C(b+1,b-1) + C(b+2,b-1) + ... + C( (n-w)-bw+b-1,b-1)
  = C(b-1,0) + C(b,1) + C(b+1,2) + C(b+2,3) + ... + C(n-(b+1)w+b-1,n-bw)
  = C(b,0) + C(b,1) + C(b+1,2) + C(b+2,3) + ... + C(n-(b+1)w+b-1,n-(b+1)w) 
  = C(b+1,1) + C(b+1,2) + C(b+2,3) + ... + C(n-(b+1)w+b-1,n-(b+1)w) 
  = C(b+2,2) + C(b+2,3) + ... + C(n-(b+1)w+b-1,n-(b+1)w)
  ...
  = C(n-(b+1)w+b-1,n-(b+1)w -1) + C(n-(b+1)w+b-1,n-(b+1)w)
  = C(n-(b+1)w+b,n-(b+1)w)
*/
  
  long double total=pow(OneMinusPa, n -(B+1)*w + 1 - B);
  total*=pow(p_alpha, B);
  
  int r=B;  
  for(int j=n-(B+1)*w+B;j>n-(B+1)*w;j--) {
    total*=j;
    total/=r; 	
    total*=OneMinusPa;  
    r--;
  } 
   
  for(int j=n-w+1;j<=n;j++) {
    long double temp=pow(OneMinusPa, j-B*w-B+1);
    temp*=pow(p_alpha, B);
    
    r=B-1;
    for(int k=j-B*w+B-1;k>j-B*w;k--) {
      temp*=k;
      temp/=r;
      temp*=OneMinusPa;   
      r--;
    }
    
    total+=temp;
  }
  return total;
}

//calculate P_s'

//(no binding site: all > aplha+1)
//(has binding site: binding site <= alpha-1, other >alpha)

long double Tools::pS2(const int alpha, const int w, const int B, const int n) {    
  
  const int rStep=1;
  
  if(alpha<=rStep && B>0)
    return 0;
  
  if(B==0){
    if(alpha<=(int)pow(4.0,w)-rStep) {
      long double p_alphaP=Tools::pAlpha(alpha+rStep,w);
      return pow(1.0-p_alphaP,n-w+1);
    }
    else
      return 0;
  }
  
  if(B>n/w)
    return 0.0;
    
  /*  
    C(b-1, b-1) + C(b,b-1) + C(b+1,b-1) + C(b+2,b-1) + ... + C( (n-w)-bw+b-1,b-1)
  = C(b-1,0) + C(b,1) + C(b+1,2) + C(b+2,3) + ... + C(n-(b+1)w+b-1,n-bw)
  = C(b,0) + C(b,1) + C(b+1,2) + C(b+2,3) + ... + C(n-(b+1)w+b-1,n-(b+1)w) 
  = C(b+1,1) + C(b+1,2) + C(b+2,3) + ... + C(n-(b+1)w+b-1,n-(b+1)w) 
  = C(b+2,2) + C(b+2,3) + ... + C(n-(b+1)w+b-1,n-(b+1)w)
  ...
  = C(n-(b+1)w+b-1,n-(b+1)w -1) + C(n-(b+1)w+b-1,n-(b+1)w)
  = C(n-(b+1)w+b,n-(b+1)w)
*/
  long double p_alpha=Tools::pAlpha(alpha-rStep,w);
  long double OneMinusPa=1.0-Tools::pAlpha(alpha,w);;
  
  long double total=pow(OneMinusPa, n -(B+1)*w + 1 - B);
  total*=pow(p_alpha, B);
  
  int r=B;  
  for(int j=n-(B+1)*w+B;j>n-(B+1)*w;j--) {
    total*=j;
    total/=r; 	
    total*=OneMinusPa;  
    r--;
  } 
   
  for(int j=n-w+1;j<=n;j++) {
    long double temp=pow(OneMinusPa, j-B*w-B+1);
    temp*=pow(p_alpha, B);
    
    r=B-1;
    for(int k=j-B*w+B-1;k>j-B*w;k--) {
      temp*=k;
      temp/=r;
      temp*=OneMinusPa;   
      r--;
    }
    
    total+=temp;
  }
  return total;
}

//calculate C(n,r)                          
double Tools::choose(const int n, const int r) {
  if(r>n)
    return 0;
    
  if(r==0||r==n)
    return 1;
  
  double total=(double)n/r;
  for(int i=1;i<r;i++)
    total=total*(n-i)/i;
  return total;
}

//calculate n!/a_i!
long double Tools::countChain(const int n, const int* aiArr
              		      , const long double* pArr, const int noOfai){
  //calculate the value
  long double total=1.0;
  int temp=0;
  int count=-1;
  
  for(int i=n;i>=1;i--) {
    total*=(long double)i;    
    
    while(temp==0){
      count++;
      temp=aiArr[count];
    }
    
    total/=(double)temp;
    total*=pArr[count];
    temp--;
  }
  return total;
}

//find all possible distribution of binding site
void Tools::BSDistribute(int* distributeArr, const int pos, const int m
                         , const int totalSite, const int seqLeave
                         , vector<int*> &distributeVec) {
  if(pos==m) {
    int temp=0;
    for(int i=1;i<m;i++)
      temp+=i*distributeArr[i];
    temp+=m*seqLeave;
    
    if(temp==totalSite) {
      int* tempDArr=new int[m+1];
      for(int i=0;i<m;i++)
        tempDArr[i]=distributeArr[i];
      tempDArr[m]=seqLeave;
      distributeVec.push_back(tempDArr);
    }
    
  } else {
    for(int i=0;i<=seqLeave;i++) {
      distributeArr[pos]=i;
      BSDistribute(distributeArr, pos+1, m, totalSite, seqLeave-i, distributeVec);
    }
  }
}

/*****************************************************************************/
/*                                                                           */
/* Pri 1 Sorting                                                             */
/*                                                                           */
/*****************************************************************************/

//sorting the double array inArr in decreasing order (Merge sort)
double* Tools::sortDouble_D_M(double* inArr, double* tempArr
                             , int start, int end) {
  if(end-start <= 10) {
    //bubble sort
    double temp;
    for(int i=start;i<end-1;i++) {
      for(int j=end-1;j>i;j--) {
        if(inArr[j]>inArr[j-1]) {
          temp=inArr[j-1];
          inArr[j-1]=inArr[j];
          inArr[j]=temp;
        }
      }
    }
    return inArr;
  }
  else {
    //merge sort
    int mid=(end+start)/2;
    
    double* firstHalf=Tools::sortDouble_D_M(inArr,tempArr,start,mid);
    double* secondHalf=Tools::sortDouble_D_M(inArr,tempArr,mid,end);
    
    double* out;
    if(firstHalf==inArr)
      out=tempArr;
    else
      out=inArr;
      
    int count=start;
    int firstStart=start;
    int secondStart=mid;
    
    while(firstStart < mid && secondStart < end) {
      if(firstHalf[firstStart] > secondHalf[secondStart]) {
        out[count]=firstHalf[firstStart];        
        firstStart++;
        count++;
      }
      else {
      	out[count]=secondHalf[secondStart];
        secondStart++;
        count++;
      }
    }    
    if(firstStart<mid) {
      while(count<end) {
        out[count]=firstHalf[firstStart];
        count++;
        firstStart++;
      }
    }
    else {
      while(count<end) {
        out[count]=secondHalf[secondStart];
        count++;
        secondStart++;
      }
    }
    
    return out;
  }
}

BindingSitePosition** Tools::sortBSP_D_M(BindingSitePosition** inArr
                                         , BindingSitePosition** tempArr
                                         , int start, int end) {
  if(end-start <= 10) {
    //bubble sort
    BindingSitePosition* temp;
    for(int i=start;i<end-1;i++) {
      for(int j=end-1;j>i;j--) {
        if(inArr[j]->getScore() > inArr[j-1]->getScore()) {
          temp=inArr[j-1];
          inArr[j-1]=inArr[j];
          inArr[j]=temp;
        }
      }
    }
    return inArr;
  }
  else {
    //merge sort
    int mid=(end+start)/2;
    
    BindingSitePosition** firstHalf=Tools::sortBSP_D_M(inArr,tempArr,start,mid);
    BindingSitePosition** secondHalf=Tools::sortBSP_D_M(inArr,tempArr,mid,end);
    
    BindingSitePosition** out;
    if(firstHalf==inArr)
      out=tempArr;
    else
      out=inArr;
      
    int count=start;
    int firstStart=start;
    int secondStart=mid;
    
    while(firstStart < mid && secondStart < end) {
      if(firstHalf[firstStart]->getScore() > secondHalf[secondStart]->getScore()) {
        out[count]=firstHalf[firstStart];        
        firstStart++;
        count++;
      }
      else {
      	out[count]=secondHalf[secondStart];
        secondStart++;
        count++;
      }
    }    
    if(firstStart<mid) {
      while(count<end) {
        out[count]=firstHalf[firstStart];
        count++;
        firstStart++;
      }
    }
    else {
      while(count<end) {
        out[count]=secondHalf[secondStart];
        count++;
        secondStart++;
      }
    }
    
    return out;
  }
}

/*****************************************************************************/
/*                                                                           */
/* Main function                                                             */
/*                                                                           */
/*****************************************************************************/

/*
int main(int argc, char *argv[]) {	
  int T=50;
  int F=500;
  int w=7;
  double m=2.0;
  int n=500;
  
  long start=time(0);
  Tools t(T, F, m, w,n);
  cout<<">>>Total Time taken = "<< (time(0)-start)/60<<"m"<<endl;
}
*/


//find all combination of (comArrSize-comArrPos) balls into
//(m-combinationArr[comArrPos-1]-1) boxes
void Tools::combination(long* combinationArr, const long comArrSize, const long comArrPos
                        , const long m, vector<long*> &comArrVec) {
  if(comArrPos==comArrSize) {
    long* tempComArr=new long[comArrSize];
    for(long i=0;i<comArrSize;i++)
      tempComArr[i]=combinationArr[i];
    comArrVec.push_back(tempComArr);
  } else {
    //check if no of boxes = no of balls
    long boxLeave=m;
    long boxStart=0;
    long noOfBalls=comArrSize-comArrPos;
    
    if(comArrPos!=0){
      boxStart=combinationArr[comArrPos-1]+1;
      boxLeave-=boxStart;
    }
    
    if(boxLeave==noOfBalls) {
      for(long i=0;i<noOfBalls;i++)
        combinationArr[comArrPos+i]=boxStart+i;
      
      long* tempComArr=new long[comArrSize];
      for(long i=0;i<comArrSize;i++)
        tempComArr[i]=combinationArr[i];
      comArrVec.push_back(tempComArr);
    } else {
      for(long i=0;i<=boxLeave-noOfBalls;i++) {
        combinationArr[comArrPos]=boxStart+i;
        combination(combinationArr, comArrSize, comArrPos+1,m,comArrVec);
      }
    }
  }
}

/*****************************************************************************/
/* String operation                                                          */
/*****************************************************************************/

void Tools::longToPattern(char* pattern, const long patternLength
                          , const unsigned long number) {
  unsigned long tempNum=number;
  for(long i=0;i<patternLength;i++) {
    switch(tempNum%4) {
      case 0:
        pattern[i]='A';
        break;
      case 1:
        pattern[i]='C';
        break;
      case 2:
        pattern[i]='G';
        break;
      case 3:
        pattern[i]='T';
        break;
    }
    tempNum/=4;
  }
  pattern[patternLength]=0;
}

unsigned long Tools::patternToLong(const char* pattern, const long patternLength) {

  unsigned long number=0;
  for(int i=0;i<patternLength;i++) {
    number*=4;
    switch(pattern[patternLength-1-i]){
      case 'A':
        //number+=0;
        break;
      case 'C':
        number+=1;
        break;
      case 'G':
        number+=2;
        break;
      case 'T':
        number+=3;
        break;
      default:
        cout << "In Tools::patternToLong" << endl;
        cout << "The " << patternLength-1-i << "-th character in pattern is \'"<<pattern[patternLength-1-i]
            <<"\' which is not a valid character"<<endl;
        exit(1);
        break;  
    }
  }
  return number;
}

long Tools::calHamDist(const char* pattern1, const char* pattern2
                       , const long patternLength) {
  long hamDist=0;
  for(long i=0;i<patternLength;i++)
    if(pattern1[i]!=pattern2[i])
      hamDist++;
  return hamDist;
}

