#ifndef TOOLS_H
#define TOOLS_H
#include "header.h"
 
class Tools {
public:
  Tools(const int T, const int F, const double m, const int w, const int n);	//constructor
  ~Tools();	//destructor
  
/*****************************************************************************/
/*                                                                           */
/* Pub 1 Sorting                                                             */
/*                                                                           */
/*****************************************************************************/

  //sorting the double array inArr in decreasing order
  static void sortDouble_D(double* inArr, const int arraySize);
  
  //sorting the BindingSitePosition array inArr in decreasing order
  static void sortBSP_D(BindingSitePosition** inArr, const int arraySize);
  
/*****************************************************************************/
/*                                                                           */
/* Pub 2 String Operation Methods                                            */
/*                                                                           */
/*****************************************************************************/

  //convert an integer to a string  
  static string intToStr(int i);
  
  //convert a double to a string  
  static string dblToStr(double d);
  
  //split a string str into pieces
  static void split(string str, char delimiter, vector<string> &tokenVec);
  
  //convert a probability matrix to string
  static void matToStr(const double *probMat, char* motif, const int motifLength);

  //convert a number to probability matrix
  static void numToMat(const int num, double* probMat, const int motifLength, const int seedLength);
  
  //construct an array of all string of length strinfLength
  static void allString(char** stringArr, const int stringLength);
  
/*****************************************************************************/
/*                                                                           */
/* Pub 3 probability of a random matrix with higer t and lower f             */
/*                                                                           */
/*****************************************************************************/
  
  //calculate the score
  double score(const int t, const int f); 
  
  //calculate the p_d
  static long double pAlpha(const int alpha, const int w);

  //calculate P_s
  static long double pS(const int alpha, const int w, const int B, const int n);
  
  //calculate P_s' (binding site <= alpha-1, other >alpha)
  static long double pS2(const int alpha, const int w, const int B, const int n);
  	
  //calculate C(n,r)
  static double choose(const int n, const int r);
  
  //calculate n!/a_i!
  static long double countChain(const int n, const int* aiArr
                                , const long double* pArr, const int noOfai);
  
  //find all possible distributions for no of binding sites
  static void BSDistribute(int* distributeArr, const int pos, const int m
                           , const int totalSite, const int seqLeave
                           , vector<int*> &distributeVec);


static long double choose(const long n, const long r);

static void combination(long* combinationArr, const long comArrSize, const long comArrPos

                        , const long m, vector<long*> &comArrVec);  



static void longToPattern(char* pattern, const long patternLength

                          , const unsigned long number);

static unsigned long patternToLong(const char* pattern, const long patternLength);

static long calHamDist(const char* pattern1, const char* pattern2, const long patternLength);



//convert an integer to a string
static string intToStr(int i);

                           
  private:
/*****************************************************************************/
/*                                                                           */
/* Pri 1 Sorting                                                             */
/*                                                                           */
/*****************************************************************************/

  //sorting the double array inArr in decreasing order (Merge sort)
  static double* sortDouble_D_M(double* inArr, double* tempArr
                             , int start, int end);
  
   //sorting the BindingSitePosition array inArr in decreasing order (Merge sort)
  static BindingSitePosition** sortBSP_D_M(BindingSitePosition** inArr
                                           , BindingSitePosition** tempArr
                                           , int start, int end);

/*****************************************************************************/
/*                                                                           */
/* Pri 2 Parameters                                                            */
/*                                                                           */
/*****************************************************************************/
  int T;
  int F;
  int motifLength;
  double maxBindRatio;
  int seqLength;
  
  int noOfRowT;
  int noOfColF;
  long double* expectArr;
};

#endif
